package org.homenet.ehiahi.whereisthounet;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import org.json.JSONException;

import android.os.Environment;

public class BufferLoggerThread extends Thread {

	private static final int SCAN_INTERVAL = 2000;
	private WhereIsThouNetActivity activity;
	private volatile boolean is_running = true;
	private volatile boolean pause_logging = false;
	private DataBuffer info;

	public BufferLoggerThread(WhereIsThouNetActivity activity,
			DataBuffer info) {
		this.activity = activity;
		this.info = info;
	}

	@Override
	public void run() {
		super.run();
		String start_time = now().replace(":", "_");
		String data_folder_path = Environment.getExternalStorageDirectory().getAbsolutePath()+"/download/wtn";
		File folder = new File(data_folder_path);
		if ( !folder.exists() ) folder.mkdir();
		activity.logMessage("BLThread: started");
		while ( is_running ) {
			try {
				sleep(SCAN_INTERVAL);
				if ( pause_logging ) continue;
				if ( Environment.MEDIA_MOUNTED.equals(Environment.getExternalStorageState()) &&
						info.wasUpdated() ) {
					File file = new File(
						data_folder_path, start_time+".txt"
					);
					FileWriter fstream = new FileWriter(file, true);
					BufferedWriter out = new BufferedWriter(fstream);
					out.write(info.toJson(now())+"\n");
					out.close();
				} else activity.logMessage("BLThread: data cannot be logged");
			} catch (InterruptedException e) {
				activity.logMessage("BLThread: sleep interrupted");
			} catch (IOException e) {
				activity.logMessage("BLThread: error saving data "+e.getMessage());
			} catch (JSONException e) {
				activity.logMessage("BLThread: error creating json");
			}
		}
		activity.logMessage("BLThread: stopped");
	}

	private static String now() {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		return sdf.format(cal.getTime());
	}

	public void stopExecution() {
		is_running = false;
	}

	public void pause() {
		pause_logging = true;
	}

	public void unpause() {
		pause_logging = false;
	}

}
