package org.homenet.ehiahi.whereisthounet;

import java.text.SimpleDateFormat;
import java.util.Calendar;

import android.app.Activity;
import android.content.Context;
import android.content.IntentFilter;
import android.location.LocationManager;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;

public class WhereIsThouNetActivity extends Activity {

    private WIFIScanInducerThread scan_inducer_thread = null;
	private BufferLoggerThread buffer_logger_thread = null;
	private DataBuffer info;
	private GWLLocationListener location_listener;
	private WiFiScanReceiver receiver;

	@Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        info = new DataBuffer();
		receiver = new WiFiScanReceiver(
			this, (WifiManager) getSystemService(
				Context.WIFI_SERVICE
			), info
		);
	    registerReceiver(receiver, new IntentFilter(
		    WifiManager.SCAN_RESULTS_AVAILABLE_ACTION
	    ));
	    startScanInducerThread();
		startLocationListener();
    }

	@Override
    public void onDestroy() {
    	super.onDestroy();
   		stopScanInducerThread();
   		stopLocationListener();
    	if ( buffer_logger_thread != null )
    		stopBufferLoggerThread();
	    unregisterReceiver(receiver);
    }

	public void updateGPS(final double latitude, final double longitude,
			final double altitude) {
		runOnUiThread(new Runnable() { 
			public void run() {
				((EditText) findViewById(R.id.latitude)).setText(latitude+" ("+now()+")");
				((EditText) findViewById(R.id.longitude)).setText(longitude+" ("+now()+")");
				((EditText) findViewById(R.id.altitude)).setText(altitude+" ("+now()+")"); 
			}
		});
	}

	public void updateWIFI(final int quantity, final String networks_list) {
		runOnUiThread(new Runnable() { 
			public void run() {
				((EditText) findViewById(R.id.networks)).setText(quantity+" ("+now()+")");
				((EditText) findViewById(R.id.networks_list)).setText(networks_list+" ("+now()+")"); 
			}
		});
	}

	public void logMessage(final String message) {
		runOnUiThread(new Runnable() { 
			public void run() {
				EditText edit_text = (EditText) findViewById(R.id.log_area);
				edit_text.append(now()+"|"+message+"\n"); 
			}
		});
	}

	public void recButton(View object) {
		Button button = (Button) object;
		button.setEnabled(false);
		((Button) findViewById(R.id.pause)).setEnabled(true);
		((Button) findViewById(R.id.stop)).setEnabled(true);
		startBufferLoggerThread();
	}

	public void pauseButton(View object) {
		Button button = (Button) object;
		if ( button.getText().equals(getString(R.string.pause)) ) {
			button.setText(getString(R.string.paused));
			buffer_logger_thread.pause();
		} else {
			button.setText(getString(R.string.pause));
			buffer_logger_thread.unpause();
		}
	}

	public void stopButton(View object) {
		Button button = (Button) object;
		((Button) findViewById(R.id.rec)).setEnabled(true);
		button.setEnabled(false);
		((Button) findViewById(R.id.pause)).setEnabled(false);
		((Button) findViewById(R.id.pause)).setText(
			getString(R.string.pause)
		);
		stopBufferLoggerThread();
	}

	private static String now() {
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(
			Calendar.getInstance().getTime()
		);
	}

    private void startScanInducerThread() {
		scan_inducer_thread = new WIFIScanInducerThread(
			this, receiver
		);
		scan_inducer_thread.start();
	}

    private void stopScanInducerThread() {
    	scan_inducer_thread.stopExecution();
    	scan_inducer_thread.interrupt();
    	scan_inducer_thread = null;
	}

    private void startBufferLoggerThread() {
		buffer_logger_thread = new BufferLoggerThread(
			this, info
		);
		buffer_logger_thread.start();
	}

    private void stopBufferLoggerThread() {
    	buffer_logger_thread.stopExecution();
    	buffer_logger_thread.interrupt();
    	buffer_logger_thread = null;
	}

	private void startLocationListener() {
        location_listener = new GWLLocationListener(
        	this, info, (LocationManager) getSystemService(
        		Context.LOCATION_SERVICE
        	)
        );
	}

	private void stopLocationListener() {
        location_listener.stop();
        location_listener = null;
	}

}
